@@ -42,6 +42,7 @@ typedef struct {
   SCM get_options_lambda;
   SCM apply_options_lambda;
   SCM timeout_lambda;
+  SCM droppable_lambda;
 } lambda_data;
 
 lambda_data* game_data = NULL;
@@ -57,11 +58,11 @@ void add_slot(SCM slot_data)
   hslot->x = scm_num2dbl (SCM_CAR(SCM_CADR(SCM_CADDR(slot_data))), NULL);
   hslot->y = scm_num2dbl (SCM_CADR(SCM_CADR(SCM_CADDR(slot_data))), NULL);
   
-  hslot->dx = hslot->dy = 0;
+  hslot->dx = hslot->dy = hslot->compressed_dy = 0;
   hslot->expansion_depth = 0;
 
   if (!strcmp("expanded", SCM_CHARS(SCM_CAR(SCM_CADDR(slot_data))))) {
-    hslot->dy = y_expanded_offset;
+    hslot->compressed_dy = hslot->dy = y_expanded_offset;
   }
   else if (!strcmp("expanded-right", 
 		   SCM_CHARS(SCM_CAR(SCM_CADDR(slot_data))))) {
@@ -69,7 +70,7 @@ void add_slot(SCM slot_data)
   }
   else if (!strcmp("partially-expanded", 
 		   SCM_CHARS(SCM_CAR(SCM_CADDR(slot_data))))) {
-    hslot->dy = y_expanded_offset;
+    hslot->compressed_dy = hslot->dy = y_expanded_offset;
     hslot->expansion_depth = SCM_INUM (SCM_CADDR(SCM_CADDR(slot_data)));
   }
   else if (!strcmp("partially-expanded-right", 
@@ -158,10 +159,26 @@ static SCM scm_redo_set_sensitive (SCM in_state)
   return SCM_EOL;
 }
 
+static SCM scm_get_feature_word()
+{
+  return SCM_MAKINUM(enabled_features);
+}
+
+static SCM scm_set_feature_word(SCM features)
+{
+  enabled_features = SCM_INUM(features);
+
+  droppable_is_featured = (enabled_features & DROPPABLE_FMASK);
+  score_is_hidden = (enabled_features & SCOREHIDDEN_FMASK);
+
+  return SCM_EOL;
+}
+
 static SCM scm_set_statusbar_message(SCM message)
 {
-  gnome_appbar_clear_stack (GNOME_APPBAR (GNOME_APP (app)->statusbar));
-  gnome_appbar_push (GNOME_APPBAR(GNOME_APP (app)->statusbar), _(SCM_STRING_CHARS(message)));
+  guint context_id = gtk_statusbar_get_context_id (GTK_STATUSBAR (statusbar), "message");
+  gtk_statusbar_pop (GTK_STATUSBAR (statusbar), context_id);
+  gtk_statusbar_push (GTK_STATUSBAR (statusbar), context_id, _(SCM_STRING_CHARS(message)));
   return SCM_EOL;
 }
 
@@ -194,7 +211,7 @@ static SCM gg_scm_add_slot(SCM slot)
 static SCM scm_set_slot_y_expansion(SCM scm_slot_id, SCM new_exp_val)
 {
   hslot_type slot = get_slot(scm_num2int(scm_slot_id, SCM_ARG1, NULL));
-  slot->dy = scm_num2dbl (new_exp_val, NULL);
+  slot->compressed_dy = slot->dy = scm_num2dbl (new_exp_val, NULL);
   return SCM_EOL;
 }
 
@@ -257,6 +274,9 @@ static SCM scm_set_lambda(SCM start_game_lambda,
   game_data->apply_options_lambda = SCM_CADR(rest);
   game_data->timeout_lambda = SCM_CADDR(rest);
 
+  if (droppable_is_featured)
+    game_data->droppable_lambda = SCM_CADDDR(rest);
+
   return SCM_EOL;
 }
 
@@ -265,7 +285,12 @@ static SCM scm_myrandom(SCM range)
   return scm_long2num(g_random_int_range(0,SCM_INUM(range)));
 }
 
-static SCM scm_get_score() 
+static SCM scm_click_to_move_p (void)
+{
+  return click_to_move ? SCM_BOOL_T : SCM_BOOL_F;
+}
+
+static SCM scm_get_score(void) 
 {
   return scm_long2num(score);
 }
@@ -297,6 +322,8 @@ static SCM scm_get_timeout ()
 
 void cscm_init () 
 {
+  scm_c_define_gsubr("set-feature-word!", 1, 0, 0, scm_set_feature_word);
+  scm_c_define_gsubr("get-feature-word", 0, 0, 0, scm_get_feature_word);
   scm_c_define_gsubr("set-statusbar-message", 1, 0, 0, scm_set_statusbar_message);
   scm_c_define_gsubr("set-surface-layout", 1, 0, 0, scm_set_surface_layout);
   scm_c_define_gsubr("reset-surface", 0, 0, 0, scm_reset_surface);
@@ -307,6 +334,7 @@ void cscm_init ()
   scm_c_define_gsubr("set-slot-x-expansion!", 2, 0, 0, scm_set_slot_x_expansion);
   scm_c_define_gsubr("set-lambda", 8, 0, 1, scm_set_lambda);
   scm_c_define_gsubr("random", 1, 0, 0, scm_myrandom);
+  scm_c_define_gsubr("click-to-move?", 0, 0, 0, scm_click_to_move_p);
   scm_c_define_gsubr("get-score", 0, 0, 0, scm_get_score);  
   scm_c_define_gsubr("set-score!", 1, 0, 0, scm_set_score);
   scm_c_define_gsubr("get-timeout", 0, 0, 0, scm_get_timeout);  
@@ -494,6 +522,7 @@ SCM
 cscmi_start_game_lambda (void)
 {
   CallData *call_data;
+  SCM retval;
 
   call_data = g_new0 (CallData, 1);
   call_data->lambda = game_data->start_game_lambda;
@@ -503,9 +532,10 @@ cscmi_start_game_lambda (void)
 		      call_data,
 		      cscmi_catch_handler,
 		      NULL);
+  retval = call_data->retval;
   g_free (call_data);
 
-  return call_data->retval;
+  return retval;
 }
 
 SCM
@@ -513,6 +543,7 @@ cscmi_button_pressed_lambda (SCM slot_id,
 			     SCM cards)
 {
   CallData *call_data;
+  SCM retval;
 
   call_data = g_new0 (CallData, 1);
   call_data->lambda = game_data->button_pressed_lambda;
@@ -524,9 +555,10 @@ cscmi_button_pressed_lambda (SCM slot_id,
 		      call_data,
 		      cscmi_catch_handler,
 		      NULL);
+  retval = call_data->retval;
   g_free (call_data);
 
-  return call_data->retval;
+  return retval;
 }
 
 SCM
@@ -535,6 +567,7 @@ cscmi_button_released_lambda (SCM start_slot,
 			      SCM end_slot)
 {
   CallData *call_data;
+  SCM retval;
 
   call_data = g_new0 (CallData, 1);
   call_data->lambda = game_data->button_released_lambda;
@@ -547,15 +580,17 @@ cscmi_button_released_lambda (SCM start_slot,
 		      call_data,
 		      cscmi_catch_handler,
 		      NULL);
+  retval = call_data->retval;
   g_free (call_data);
 
-  return call_data->retval;
+  return retval;
 }
 
 SCM
 cscmi_button_clicked_lambda (SCM slot_id)
 {
   CallData *call_data;
+  SCM retval;
 
   call_data = g_new0 (CallData, 1);
   call_data->lambda = game_data->button_clicked_lambda;
@@ -566,15 +601,17 @@ cscmi_button_clicked_lambda (SCM slot_id)
 		      call_data,
 		      cscmi_catch_handler,
 		      NULL);
+  retval = call_data->retval;
   g_free (call_data);
 
-  return call_data->retval;
+  return retval;
 }
 
 SCM
 cscmi_button_double_clicked_lambda (SCM slot_id)
 {
   CallData *call_data;
+  SCM retval;
 
   call_data = g_new0 (CallData, 1);
   call_data->lambda = game_data->button_double_clicked_lambda;
@@ -585,15 +622,45 @@ cscmi_button_double_clicked_lambda (SCM slot_id)
 		      call_data,
 		      cscmi_catch_handler,
 		      NULL);
+  retval = call_data->retval;
   g_free (call_data);
 
-  return call_data->retval;
+  return retval;
+}
+
+SCM
+cscmi_droppable_lambda (SCM start_slot,
+                        SCM cards,
+                        SCM end_slot)
+{
+  CallData *call_data;
+  SCM retval;
+
+  if (!droppable_is_featured) 
+    return SCM_BOOL_F;
+  
+  call_data = g_new0 (CallData, 1);
+  call_data->lambda = game_data->droppable_lambda;
+  call_data->n_args = 3;
+  call_data->arg1 = start_slot;
+  call_data->arg2 = cards;
+  call_data->arg3 = end_slot;
+  scm_internal_catch (SCM_BOOL_T,
+                      cscmi_call_lambda,
+                      call_data,
+                      cscmi_catch_handler,
+                      NULL);
+  retval = call_data->retval;
+  g_free (call_data);
+
+  return retval;
 }
 
 SCM
 cscmi_game_over_lambda (void)
 {
   CallData *call_data;
+  SCM retval;
 
   call_data = g_new0 (CallData, 1);
   call_data->lambda = game_data->game_over_lambda;
@@ -603,15 +670,17 @@ cscmi_game_over_lambda (void)
 		      call_data,
 		      cscmi_catch_handler,
 		      NULL);
+  retval = call_data->retval;
   g_free (call_data);
 
-  return call_data->retval;
+  return retval;
 }
 
 SCM
 cscmi_winning_game_lambda (void)
 {
   CallData *call_data;
+  SCM retval;
 
   call_data = g_new0 (CallData, 1);
   call_data->lambda = game_data->winning_game_lambda;
@@ -621,15 +690,17 @@ cscmi_winning_game_lambda (void)
 		      call_data,
 		      cscmi_catch_handler,
 		      NULL);
+  retval = call_data->retval;
   g_free (call_data);
 
-  return call_data->retval;
+  return retval;
 }
 
 SCM
 cscmi_hint_lambda (void)
 {
   CallData *call_data;
+  SCM retval;
 
   call_data = g_new0 (CallData, 1);
   call_data->lambda = game_data->hint_lambda;
@@ -639,15 +710,17 @@ cscmi_hint_lambda (void)
 		      call_data,
 		      cscmi_catch_handler,
 		      NULL);
+  retval = call_data->retval;
   g_free (call_data);
 
-  return call_data->retval;
+  return retval;
 }
 
 SCM
 cscmi_get_options_lambda (void)
 {
   CallData *call_data;
+  SCM retval;
 
   call_data = g_new0 (CallData, 1);
   call_data->lambda = game_data->get_options_lambda;
@@ -657,15 +730,26 @@ cscmi_get_options_lambda (void)
 		      call_data,
 		      cscmi_catch_handler,
 		      NULL);
+  retval = call_data->retval;
   g_free (call_data);
 
-  return call_data->retval;
+  return retval;
+}
+
+gboolean
+has_options (void)
+{
+  SCM options;
+
+  options = cscmi_get_options_lambda ();
+  return SCM_NFALSEP (options);
 }
 
 SCM
 cscmi_apply_options_lambda (SCM options)
 {
   CallData *call_data;
+  SCM retval;
 
   call_data = g_new0 (CallData, 1);
   call_data->lambda = game_data->apply_options_lambda;
@@ -676,15 +760,17 @@ cscmi_apply_options_lambda (SCM options)
 		      call_data,
 		      cscmi_catch_handler,
 		      NULL);
+  retval = call_data->retval;
   g_free (call_data);
 
-  return call_data->retval;
+  return retval;
 }
 
 SCM
 cscmi_timeout_lambda (void)
 {
   CallData *call_data;
+  SCM retval;
 
   call_data = g_new0 (CallData, 1);
   call_data->lambda = game_data->timeout_lambda;
@@ -694,7 +780,8 @@ cscmi_timeout_lambda (void)
 		      call_data,
 		      cscmi_catch_handler,
 		      NULL);
+  retval = call_data->retval;
   g_free (call_data);
 
-  return call_data->retval;
+  return retval;
 }
